package it.unipd.app;

/**
 * 
 * @author nicola
 *
 */
public class BowlingGame implements BowlingGameInterface {
    /**
     * Number of frames
     */
    protected static final int FRAMES_NUMBER = 10;

   /**
    * Number of pins
    */
    protected static final int PINS_NUMBER = 10;

    /**
     * Number of rolls per frames 
     */
    protected static final int ROLLS_NUMBER = 2;

    /**
     * Game's frames
     */
    private Frame[] frames = new Frame[FRAMES_NUMBER];
    
    /**
     * Current position in the game's frame
     */
    private int currentFrame = 0;

    /**
     * 
     */
    public BowlingGame() {
        for (int i = 0; i < frames.length - 1; i++)
            frames[i] = new Frame(ROLLS_NUMBER,PINS_NUMBER);
        frames[frames.length - 1] = new LastFrame(ROLLS_NUMBER, PINS_NUMBER);
    }

    /* (non-Javadoc)
     * @see it.unipd.app.BowlingGameInterface#roll(int)
     */
    public void roll(final int pins) {
        frames[currentFrame].roll(pins);
        if (frames[currentFrame].finish()) {
            currentFrame ++ ;
        }
    }

    /* (non-Javadoc)
     * @see it.unipd.app.BowlingGameInterface#score()
     */
    public int score() {
        if (!finish()) {
            throw new RuntimeException("The game doesn't still finish");
        }
        int score = 0;
        for (int frame = 0; frame < frames.length; frame++) {
            if (frames[frame].isStrike()) {
                score += strikeBonus(frame);
            } else {
                if (frames[frame].isSpeare()) {
                    score += spareBonus(frame);
                } else {
                    score += frames[frame].sumOfPinsInFrame();
                }
            }
        }
        return score;
    }

    /* (non-Javadoc)
     * @see it.unipd.app.BowlingGameInterface#finish()
     */
    public boolean finish() {
        return currentFrame == frames.length
        && frames[currentFrame - 1].finish();
    }

    /* (non-Javadoc)
     * @see it.unipd.app.BowlingGameInterface#getFramePosition()
     */
    public int getFramePosition() {
        return currentFrame;
    }

    private int spareBonus(int frameIndex) {
        int nextBall = 0;
        if (frameIndex == FRAMES_NUMBER -1) {
            nextBall = frames[frameIndex].getRollScore(2);
        } else {
            nextBall = frames[frameIndex + 1].getRollScore(0);
        }
        return PINS_NUMBER + nextBall;
    }

    private int strikeBonus(final int frameIndex) {
        return PINS_NUMBER + nextTwoBall(frameIndex);
    }

    private int nextTwoBall(final int frameIndex) {
        int result = 0;
        switch (frameIndex) {
        case FRAMES_NUMBER - 1:
            result = frames[frameIndex].getRollScore(1)
            + frames[frameIndex].getRollScore(2);
            break;
        case FRAMES_NUMBER - 2:
            result = frames[frameIndex + 1].getRollScore(0)
            + frames[frameIndex + 1].getRollScore(1);
            break;
        default:
            if (frames[frameIndex + 1].isStrike()) {
                result = frames[frameIndex + 1].getRollScore(0)
                + frames[frameIndex + 2].getRollScore(0);
            } else {
                result = frames[frameIndex + 1].sumOfPinsInFrame();
            }
            break;
        }
        return result;
    }
}

