package it.unipd.app;

import junit.framework.TestCase;

public class BowlingGameTest extends TestCase {

    private BowlingGameInterface game;

    protected void setUp() throws Exception {
        game = new BowlingGame();
    }

    private void rollMany(int num, int pins) {
        for (int i = 0; i < num; i++)
            game.roll(pins);
    }

    public void testAllZeros() throws Exception {
        rollMany(20,0);
        assertEquals(0, game.score());
    }

    public void testAllOnes() throws Exception {
        rollMany(20,1);
        assertEquals(20, game.score());
    }

    public void testOneSpeare() throws Exception {
        rollManySpeare(1);
        rollMany(1,3);
        rollMany(17,0);
        assertEquals(16, game.score());
    }

    public void testOneStrike() throws Exception {
        rollManyStrike(1);
        game.roll(4);
        game.roll(3);
        rollMany(16, 0);
        assertEquals(24, game.score());
      }

    public void testPerfetGame() throws Exception {
        rollManyStrike(12);
        assertEquals(300, game.score());
      }
    
    public void testAllSpeare() throws Exception {
        rollManySpeare(10);
        game.roll(5);
        assertEquals(150, game.score());
      }
    

    private void rollManyStrike(int num){
        rollMany(num,10);
    }

    private void rollManySpeare(int num) {
        for(int i = 0; i < num; i++)
        rollMany(2,5);
    }
    
    public void testNotEndGame() {
     try{
         game.score();
         fail("Should have thrown an exception");
     } catch (RuntimeException e){
         assertTrue(true);
     }
    }

    public void testNotEndPerfectGame() {
        try{
            rollManyStrike(11);
            game.score();
            fail("Should have thrown an exception");
        } catch (RuntimeException e){
            assertTrue(true);
        }
       }

    public void testPrePostConditionRoll() {
        for (int i = 0; i < 10; i++){
            assertFalse(game.finish());
            assertEquals(i,game.getFramePosition());
            game.roll(1);
            assertFalse(game.finish());
            assertEquals(i,game.getFramePosition());
            game.roll(1);
        }
        assertTrue(game.finish());
    }
}